--[[
-- Name: TinyTip
-- Author: Thrae of Maelstrom (aka "Matthew Carras")
-- Release Date: 6-25-06
--
-- Based very losely off AF_Tooltip_Mini
-- Some code from PerfectTooltip by cladhaire
--]]

local DefaultOptions = {
	["_v"]			= 5,	-- database version, not addon's, don't change this
	["NotFade"]		= nil,
	["Anchor"]		= "CURSOR",
	["OffX"]		= 0,
	["OffY"]		= 0,
	["Scale"]		= nil,
	["NotLevelText"]	= nil,
	["NotInfoLine"]		= nil,
	["NotRace"]			= nil,
	["PvPRank"]			= 2,
	["PvPIcon"]			= nil,
	["BGColor"]			= 1,
	["Border"]			= 1,
	["NotGuild"]		= nil,
	["ReactionText"]	= nil,
	["NotCivilians"]	= nil, 
	["NotFriends"]		= nil,
	["LevelGuess"]		= nil,
	["ToM"]			= true,	
	["ToP"]			= 1,	
	["ToR"]			= 1	
}

-- This will get destroyed after TinyTipUtil_InitDB()
-- Don't change this. These values in here are random.
-- Note: When adding saved variables, TinyTipChatLocale files
-- must also be changed if you're going to include them
-- as options.
local AllOptions = {
	["NotFade"]		= true,
	["Anchor"]		= "string",
	["OffX"]		= 1,
	["OffY"]		= 1,
	["Scale"]		= 1,
	["NotLevelText"]	= true,
	["NotInfoLine"]		= true,
	["NotRace"]			= true,
	["PvPRank"]			= 1,
	["PvPIcon"]			= true,
	["BGColor"]			= 1,
	["Border"]			= 1,
	["NotGuild"]		= true,
	["ReactionText"]	= true,
	["NotCivilians"]	= true,
	["NotFriends"]		= true,
	["LevelGuess"]		= true,
	["ToM"]				= true,
	["ToP"]				= 1,
	["ToR"]				= 1
}

local strformat = string.format
local strfind = string.find

--------------------------------------------------------------------
-- Local Functions

-- Goes through all options, sees what cooresponding functions
-- don't exist, and then removes them
local function MissingFunctions()
	if not TinyTipAnchor_Hook then 
		TinyTipUtil_SetDB("Anchor", nil)
		if TinyTipChatLocale_ChatMap then 
			TinyTipChatLocale_ChatMap["Anchor"] = nil 
			TinyTipChatLocale_Opt_Anchor = nil
			TinyTipChatLocale_Desc_Anchor = nil
		end
	end
	if not TinyTipIcons_AddPvPIconNHook then
		TinyTipUtil_SetDB("PvPIcon", nil)
		if TinyTipChatLocale_ChatMap then 
			TinyTipChatLocale_ChatMap["PvPIcon"] = nil 
			TinyTipChatLocale_Opt_PvPIcon = nil
			TinyTipChatLocale_Desc_PvPIcon = nil
		end
	end
	if not TinyTipTargets_Mouse then
		TinyTipUtil_SetDB("ToM", nil)
		TinyTipUtil_SetDB("ToP", nil)
		TinyTipUtil_SetDB("ToR", nil)
		if TinyTipChatLocale_ChatMap then 
			TinyTipChatLocale_ChatMap["ToM"] = nil 
			TinyTipChatLocale_Opt_ToM = nil
			TinyTipChatLocale_Desc_ToM = nil
			TinyTipChatLocale_ChatMap["ToP"] = nil 
			TinyTipChatLocale_Opt_ToP = nil
			TinyTipChatLocale_Desc_ToP = nil
			TinyTipChatLocale_ChatMap["ToR"] = nil 
			TinyTipChatLocale_Opt_ToR = nil
			TinyTipChatLocale_Desc_ToR = nil
		end
	end

	MissingFunctions = nil
end

-- meat of TinyTip
local function TooltipFormat(unit)
	local i,treac,cdeadortap,levelLine,llText,bdead

	-- First, find Level line
	-- 
	tmp = GameTooltip:NumLines()
	for i = 1,tmp,1 do
		llText = getglobal("GameTooltipTextLeft" .. i):GetText()
		if llText and strfind(llText, getglobal("LEVEL"), 1, true) then
			if i == 3 and TinyTipUtil_GetDB("NotInfoLine") then 
				GameTooltipTextLeft3:SetText("")
				i = 2 
			end
			llText = getglobal("GameTooltipTextLeft" .. i)
			levelLine = i
			break
		end
	end

	-- remove the PvP line, unless we want it there
	if levelLine and not TinyTipUtil_GetDB("ReactionText") then
		for i = levelLine+1,tmp,1 do
			tmp2 = getglobal("GameTooltipTextLeft" .. i):GetText()
			if tmp2 and strfind(tmp2, getglobal("PVP_ENABLED"), 1, true) then
				getglobal("GameTooltipTextLeft" .. i):SetText("")
				break
			end
		end
	end
	
	-- Get Rank, then Set Name (rankName, rankNumber)
	tmp,tmp2 = GetPVPRankInfo(UnitPVPRank(unit), unit)
	if UnitIsPlayer(unit) then
		if TinyTipUtil_GetDB("PvPIcon") then
			TinyTipIcons_ResetPvPIcon(tmp2)
		end
		if TinyTipUtil_GetDB("PvPRank") and tmp2 > 0 then
			if TinyTipUtil_GetDB("PvPRank") == 1 then
				GameTooltipTextLeft1:SetText( strformat("%s %s", tmp, UnitName(unit) or "Unknown" ) )
			else
				GameTooltipTextLeft1:SetText( strformat("[R%d] %s", tmp2, UnitName(unit) or "Unknown" ) )
			end
		else
			GameTooltipTextLeft1:SetText( UnitName(unit) or "Unknown" )
		end
	end

	if TinyTipUtil_GetDB("BGColor") == 3 then
			GameTooltip:SetBackdropColor(0, 0, 0)
	end
	-- Reaction coloring
	tmp,tmp2 = UnitPlayerControlled(unit), UnitReaction(unit, "player")
	if UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) then
		GameTooltipTextLeft1:SetTextColor(0.54,0.54,0.54)
		cdeadortap = "888888"
	elseif ( tmp and UnitCanAttack(unit, "player") ) or 
		   UnitIsTappedByPlayer(unit) or 
		   ( not tmp and tmp2 and tmp2 > 0 and tmp2 <= 2 ) then -- hostile
			GameTooltipTextLeft1:SetTextColor(FACTION_BAR_COLORS[tmp2 or 2].r,
											  FACTION_BAR_COLORS[tmp2 or 2].g,
											  FACTION_BAR_COLORS[tmp2 or 2].b)
			if tmp2 and tmp2 > 0 then
				treac = getglobal("FACTION_STANDING_LABEL" .. tmp2)
			else
				treac = FACTION_STANDING_LABEL2
			end
			
			if TinyTipUtil_GetDB("BGColor") and TinyTipUtil_GetDB("BGColor") ~= 3 then
				if tmp or TinyTipUtil_GetDB("BGColor") == 2 then 
					if tmp and not UnitCanAttack("player", unit) then 
						GameTooltip:SetBackdropColor(0.5, 0.2, 0.1)
					else
						GameTooltip:SetBackdropColor(0.5, 0.0, 0.0)
					end
				end
			end
	elseif ( tmp and UnitCanAttack("player",unit) ) or 
		   ( not tmp and tmp2 and tmp2 <= 4 ) then -- neutral
			GameTooltipTextLeft1:SetTextColor(FACTION_BAR_COLORS[4].r,
											  FACTION_BAR_COLORS[4].g,
											  FACTION_BAR_COLORS[4].b)
			treac = FACTION_STANDING_LABEL4
			if TinyTipUtil_GetDB("BGColor") and TinyTipUtil_GetDB("BGColor") ~= 3 and
			(tmp or TinyTipUtil_GetDB("BGColor") == 2) then
				GameTooltip:SetBackdropColor(0.5, 0.5, 0.0)
			end
	else -- friendly
		treac = FACTION_STANDING_LABEL5
		if TinyTipUtil_GetDB("BGColor") and TinyTipUtil_GetDB("BGColor") ~= 3 and
			(tmp or TinyTipUtil_GetDB("BGColor") == 2) then
			GameTooltip:SetBackdropColor(0.0, 0.0, 0.5)
		end
		if UnitIsPVP(unit) then -- friendly, PvP-enabled
			GameTooltipTextLeft1:SetTextColor(FACTION_BAR_COLORS[6].r,
											  FACTION_BAR_COLORS[6].g,
											  FACTION_BAR_COLORS[6].b)
		else
			GameTooltipTextLeft1:SetTextColor( (not tmp and FACTION_BAR_COLORS[tmp2 or 5].r) or 0,
											   (not tmp and FACTION_BAR_COLORS[tmp2 or 5].g) or 0.67,
											   (not tmp and FACTION_BAR_COLORS[tmp2 or 5].b) or 1.0)
		end
	end

	-- We like to know who our friends are.
	if not TinyTipUtil_GetDB("NotFriends") and 
	UnitIsPlayer(unit) and treac == FACTION_STANDING_LABEL5 then
		for i = 1,GetNumFriends(),1 do
			tmp,tmp2 = GetFriendInfo(i)
			if tmp and tmp2 ~= 0 and UnitName(unit) and tmp == UnitName(unit) then
				if 	TinyTipUtil_GetDB("BGColor") == 3 then
					GameTooltipTextLeft1:SetTextColor(0.58,0.0,0.83)
				else
					GameTooltip:SetBackdropColor(0.58, 0.0, 0.83)
				end
				break
			end
		end
	end
	
	-- Check for a dead unit, but try to leave out Hunter's Feign Death
	if UnitHealth(unit) <= 0 and 
	( not UnitIsPlayer(unit) or ( UnitIsPlayer(unit) and UnitIsDeadOrGhost(unit) ) ) then
		if TinyTipUtil_GetDB("BGColor") == 3 then
			GameTooltip:SetBackdropColor(0, 0, 0)
		end
		GameTooltipTextLeft1:SetTextColor(0.54,0.54,0.54)
		if levelLine then
			cdeadortap = "888888"
		end
		bdead = true
	end	
	
	-- set the color of line 2, if it's available
	if levelLine and levelLine == 3 then
		GameTooltipTextLeft2:SetTextColor( GameTooltipTextLeft1:GetTextColor() )
	end

	if levelLine then
		local clevel
		tmp2 = UnitLevel(unit) - UnitLevel("player") -- Level difficulty
		if tmp2 and UnitFactionGroup(unit) ~= UnitFactionGroup("player") then
			if tmp2 >= 5 or tmp == -1 then clevel = "FF0000"
			elseif tmp2 >= 3 then clevel = "FF6600"
			elseif tmp2 >= -2 then clevel = "FFFF00"
			elseif -tmp2 <= GetQuestGreenRange() then clevel = "00FF00"
			else clevel = "888888"
			end		
		end

		tmp2 = UnitLevel(unit) -- Level #
		tmp = TinyTipUtil_GetDB("NotLevelText")
		if tmp2 and tmp2 >= 1 then 
			llText:SetText( strformat( "|cFF%s%s %d|r", 
								cdeadortap or clevel or "FFCC00", 
								( not tmp and getglobal("LEVEL") ) or "", tmp2 ) )
		elseif TinyTipUtil_GetDB("LevelGuess") and tmp2 and tmp2 == -1 and tmp2 < 60 then
			llText:SetText( strformat( "|cFF%s%s >%d|r", 
								cdeadortap or clevel or "FFCC00", 
								( not tmp and getglobal("LEVEL") ) or "", UnitLevel("player") + 10 ) )
		else
			llText:SetText( strformat( "|cFF%s%s ??|r", 
								 cdeadortap or clevel or "FFCC00", 
								 ( not tmp and getglobal("LEVEL") ) or "" ) )
		end

		tmp2 = UnitClassification(unit) -- Elite status
		if tmp2 and tmp2 ~= "normal" and UnitHealth(unit) > 0 then
			if tmp2 == "elite" then
				llText:SetText(strformat("%s |cFF%s%s|r", 
											llText:GetText() or "", 
											cdeadortap or "FFCC00",
											getglobal("ELITE") ) )
			elseif tmp2 == "worldboss" then
				llText:SetText(strformat("%s |cFF%s%s|r", 
											llText:GetText() or "", 
											cdeadortap or "FF0000",
											getglobal("BOSS") ) )
			elseif tmp2 == "rare" then
				llText:SetText(strformat("%s |cFF%s%s|r", 
											llText:GetText() or "", 
											cdeadortap or "FF66FF",
											getglobal("ITEM_QUALITY3_DESC") ) )
			elseif tmp2 == "rareelite" then
				llText:SetText(strformat("%s |cFF%s%s|r", 
											llText:GetText() or "", 
											cdeadortap or "FFAAFF",
											TinyTipLocale_RareElite ) )
			else
				llText:SetText(strformat("%s |cFF%s%s|r", 
											llText:GetText() or "", 
											cdeadortap or "FFFFFF",
											tmp2 ) )
			end
		end

		-- Class And Race
		if UnitIsPlayer(unit) then
			llText:SetText( strformat("%s |cFF%s%s|r |cFF%s%s|r",
							llText:GetText() or "",
							cdeadortap or "DDEEAA",
							( not TinyTipUtil_GetDB("NotRace") and UnitRace(unit) ) or "", 
							cdeadortap or TinyTipUtil_ColorPlayer(unit), UnitClass(unit) or "" ) ) 
		elseif UnitPlayerControlled(unit) then 
			llText:SetText( strformat("%s |cFF%s%s|r",
							llText:GetText() or "",
							cdeadortap or "DDEEAA",
							UnitCreatureFamily(unit) or "" ) )
		else
			llText:SetText( strformat("%s |cFF%s%s|r",
							llText:GetText() or "",
							cdeadortap or "DDEEAA",
							UnitCreatureType(unit) or "" ) )
		end
	end
	
	-- add corpse/tapped line
	if cdeadortap and llText then 
		llText:SetText( strformat( "%s |cFF888888(%s)|r", 
						llText:GetText(), 
						( bdead and getglobal("CORPSE") ) or TinyTipLocale_Tapped ) )
	end

	-- Add Reaction Text and Civilian Text, if enabled
	tmp,tmp2 = TinyTipUtil_GetDB("ReactionText"), TinyTipUtil_GetDB("NotCivilians")
	if tmp and treac and not tmp2 and UnitIsCivilian(unit) then
		GameTooltip:AddLine( strformat("%s %s", treac, getglobal("CIVILIAN") ),
							 GameTooltipTextLeft1:GetTextColor() )
	elseif tmp and treac then
		GameTooltip:AddLine( treac, GameTooltipTextLeft1:GetTextColor() )
	elseif not tmp2 and UnitIsCivilian(unit) then
		GameTooltip:AddLine( getglobal("CIVILIAN"), GameTooltipTextLeft1:GetTextColor() )
	end	

	-- Add Guild info, if enabled
	tmp2 = GetGuildInfo(unit)
	if not TinyTipUtil_GetDB("NotGuild") and levelLine and tmp2 then
		tmp = strformat("<%s>", tmp2) 
		if IsInGuild() and tmp2 == GetGuildInfo("player") then
			if not TinyTipUtil_GetDB("NotFriends") and not UnitIsUnit(unit,"player") then
				GameTooltip:SetBackdropColor(0.8, 0.2, 1)
				GameTooltip:AddLine( tmp, GameTooltipTextLeft1:GetTextColor() )
			else
				GameTooltip:AddLine( tmp, 0.14, 0.55, 0.14 )
			end
		else
			GameTooltip:AddLine( tmp, GameTooltipTextLeft1:GetTextColor() )
		end
	end

	if unit == "mouseover" then
		tmp,tmp2 = GetNumRaidMembers(), GetNumPartyMembers()
		if TinyTipUtil_GetDB("ToM") then TinyTipTargets_Mouse(unit) end
		if TinyTipUtil_GetDB("ToR") and tmp and tmp > 0 then TinyTipTargets_Raid(unit, tmp2, TinyTipUtil_GetDB("ToR"))
		elseif TinyTipUtil_GetDB("ToP") and tmp2 and tmp2 > 0 then TinyTipTargets_Party(unit, tmp2, TinyTipUtil_GetDB("ToP")) end
	end

	if TinyTipUtil_GetDB("Border") then
		if TinyTipUtil_GetDB("Border") == 2 then
			GameTooltip:SetBackdropBorderColor( 0,0,0,0 )
		else
			local r,g,b = GameTooltip:GetBackdropColor()
			r = r * 0.5
			g = g * 0.5
			b = b * 0.5
			GameTooltip:SetBackdropBorderColor( GameTooltip:GetBackdropColor() )
		end
	end

	-- If we had any extra lines from above, add them here
	GameTooltip:Show()
end

local Original_GameTooltip_SetUnit, lastText, lastUnit
local function SetUnit(this,unit)
	Original_GameTooltip_SetUnit(this,unit)
	lastUnit = unit
	TooltipFormat(unit)
	lastText = GameTooltipTextLeft1:GetText()
end

-- uber hack for disabling fade or removing PvPIcon and other textures
local lastUpdate = 0
local function OnUpdate()
	if lastUpdate + 0.2 < GetTime() then
		if lastUnit and not UnitExists(lastUnit) and GameTooltip:IsVisible() and 
		lastText == GameTooltipTextLeft1:GetText() then
			if TinyTipUtil_GetDB("NotFade") then GameTooltip:Hide() end
			if TinyTipIcons_PvPTexture then TinyTipIcons_PvPTexture:Hide() end
			TinyTipUtil_Tooltip:SetScript("OnUpdate", nil)
			lastText, lastUnit = nil
		elseif lastText ~= GameTooltipTextLeft1:GetText() then
			TinyTipUtil_Tooltip:SetScript("OnUpdate", nil)
			lastText, lastUnit = nil
		end
		lastUpdate = GetTime()
	end
end

local function OnEvent()	
	if event == "UPDATE_MOUSEOVER_UNIT" then
		if UnitExists("mouseover") then
			GameTooltip:SetUnit("mouseover")
			if TinyTipUtil_GetDB("NotFade") or TinyTipIcons_PvPTexture then
				TinyTipUtil_Tooltip:SetScript("OnUpdate", OnUpdate)
			end
		elseif not TinyTipUtil_GetDB("NotFade") then
			GameTooltip:FadeOut()
		else
			GameTooltip:Hide()
		end
	elseif event == "ADDON_LOADED" and arg1 == "TinyTip" then
		TinyTipUtil_InitDB(DefaultOptions, AllOptions)
		MissingFunctions()
		-- hook others here
		if TinyTipUtil_GetDB("Scale") then 
			GameTooltip:SetScale( TinyTipUtil_GetDB("Scale") ) 
			TinyTipUtil_Tooltip:SetScale( TinyTipUtil_GetDB("Scale") )
		end
		if TinyTipIcons_AddPvPIconNHook and TinyTipUtil_GetDB("PvPIcon") then 
			TinyTipIcons_AddPvPIconNHook()
		end
		if TinyTipAnchor_Hook and TinyTipUtil_GetDB("Anchor") then
			TinyTipAnchor_Hook()
		end
		
		TinyTipUtil_Tooltip:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
		AllOptions = nil -- no longer need this
	end
end

-----------------------------------------------------------------------
-- Ready the mod
--
		
-- Initialization code
local function tt_initialize()
	CreateFrame("GameTooltip", "TinyTipUtil_Tooltip", getglobal("UIParent"), "GameTooltipTemplate" )
	TinyTipUtil_Tooltip:SetScript("OnEvent", OnEvent)
	TinyTipUtil_Tooltip:RegisterEvent("ADDON_LOADED")
	TinyTipUtil_Tooltip:ClearAllPoints()
	TinyTipUtil_Tooltip:SetOwner(getglobal("UIParent"), "ANCHOR_CURSOR")
	TinyTipUtil_Tooltip:SetBackdropColor(0,0,0,0)
	TinyTipUtil_Tooltip:SetBackdropBorderColor(0,0,0,0)
	TinyTipUtil_Tooltip:SetText("TT", 0, 0, 0, 0)
	TinyTipUtil_Tooltip:Show()

	-- hook SetUnit, where we will format from
	Original_GameTooltip_SetUnit = GameTooltip.SetUnit
	GameTooltip.SetUnit = SetUnit

	tt_initilize = nil
end

tt_initialize()
