--[[
-- Name: TinyTip
-- Author: Thrae of Maelstrom (aka "Matthew Carras")
-- Release Date: 6-25-06
--
-- These functions allow you to change options via slash 
-- commands. You can safely delete this file (or just
-- remove it from the TOC) if you don't need to change 
-- your options anymore.
--
-- This part does NOT need to be localized, look in
-- TinyTipChatLocale_xxXX.lua.
--]]

local function ChatToggle(k)
	TinyTipUtil_ToggleDB(k)
	TinyTipUtil_Msg( string.format("%s %s [|cFFFFCC00%s|r]", 
			TinyTipChatLocale_ChatMap[k], 
			TinyTipChatLocale_IsNow,
			(TinyTipUtil_GetDB(k) and TinyTipChatLocale_On) or TinyTipChatLocale_Off) )
end

local function ChatSetNum(opt,msg,f) -- f=float
	local k, n
	_,_,k = string.find(opt, "TinyTipChatLocale_Opt_(.+)")
	if k then
		if msg ~= getglobal(opt) then
			_,_,n = string.find(msg, getglobal(opt) .. " (-?" .. ((f and ("%d" .. "%.") ) or "") .. "%d+)", 1)
			n = tonumber(n)
			if n then
				TinyTipUtil_SetDB(k, n)
				TinyTipUtil_Msg( string.format("%s %s [|cFFFFCC00%s|r]", 
				TinyTipChatLocale_ChatMap[k], 
				TinyTipChatLocale_IsNow,
				tostring( TinyTipUtil_GetDB(k) or TinyTipChatLocale_Off)) )
				return
			end
		else
			TinyTipUtil_SetDB(k,nil)
			TinyTipUtil_Msg( string.format("%s %s [|cFFFFCC00%s|r]", 
						 TinyTipChatLocale_ChatMap[k], 
						 TinyTipChatLocale_IsNow,
						 TinyTipChatLocale_Off ) )
			return
		end
	end
	
	TinyTipUtil_Msg( string.format('%s "|cFFFFCC00%s|r"', TinyTipChatLocale_InvalidInput, getglobal(opt) ) )
end

local function ChatSetString(opt,msg)
	local k,s
	_,_,k = string.find(opt, "TinyTipChatLocale_Opt_(.+)")
	if k then
		if msg ~= getglobal(opt) then
			_,_,s = string.find(msg, getglobal(opt) .. " (.+)", 1)
			if s then
				s = string.upper(s) -- we need this for TinyTipAnchor
				TinyTipUtil_SetDB(k, s)
				TinyTipUtil_Msg( string.format('%s %s [|cFFFFCC00"%s"|r]', 
				TinyTipChatLocale_ChatMap[k], 
				TinyTipChatLocale_IsNow,
				TinyTipUtil_GetDB(k) or TinyTipChatLocale_Off))
				return
			end
		else
			TinyTipUtil_SetDB(k,nil)
			TinyTipUtil_Msg( string.format("%s %s [|cFFFFCC00%s|r]", 
						 TinyTipChatLocale_ChatMap[k], 
						 TinyTipChatLocale_IsNow,
						 TinyTipChatLocale_Off ) )
			return
		end
	end
	TinyTipUtil_Msg( string.format('%s "|cFFFFCC00%s|r"', TinyTipChatLocale_InvalidInput, getglobal(opt) ) )
end

SlashCmdList["TINYTIP"] = function(msg)
	msg = string.lower(msg)
	if msg == TinyTipChatLocale_Opt_NotFade then
		ChatToggle( "NotFade" )
	elseif msg == TinyTipChatLocale_Opt_NotLevelText then
		ChatToggle( "NotLevelText" )
	elseif msg == TinyTipChatLocale_Opt_NotInfoLine then
		ChatToggle( "NotInfoLine" )
	elseif msg == TinyTipChatLocale_Opt_NotRace then
		ChatToggle( "NotRace" )
	elseif TinyTipIcons_AddPvPIconNHook and msg == TinyTipChatLocale_Opt_PvPIcon then
		ChatToggle("PvPIcon")
		if TinyTipUtil_GetDB("PvPIcon") then
			TinyTipIcons_AddPvPIconNHook()
		else
			TinyTipIcons_LazyUnhook()
		end
	elseif msg == TinyTipChatLocale_Opt_NotGuild then
		ChatToggle("NotGuild")
	elseif msg == TinyTipChatLocale_Opt_ReactionText then
		ChatToggle("ReactionText")
	elseif msg == TinyTipChatLocale_Opt_NotCivilians then
		ChatToggle("NotCivilians")
	elseif msg == TinyTipChatLocale_Opt_NotFriends then
		ChatToggle("NotFriends")
	elseif msg == TinyTipChatLocale_Opt_LevelGuess then
		ChatToggle("LevelGuess")
	elseif msg == TinyTipChatLocale_Opt_ToM then
		ChatToggle("ToM")
	elseif string.find(msg, TinyTipChatLocale_Opt_BGColor, 1, true) then
		ChatSetNum("TinyTipChatLocale_Opt_BGColor", msg )
	elseif string.find(msg, TinyTipChatLocale_Opt_Border, 1, true) then
		ChatSetNum("TinyTipChatLocale_Opt_Border", msg )
	elseif msg == TinyTipChatLocale_Opt_ColorBorder then
		ChatToggle("ColorBorder")
	elseif TinyTipChatLocale_Opt_Anchor and string.find(msg, TinyTipChatLocale_Opt_Anchor, 1, true) then
		ChatSetString("TinyTipChatLocale_Opt_Anchor", msg )
		if TinyTipUtil_GetDB("Anchor") then
			TinyTipAnchor_Hook()
		end
	elseif TinyTipChatLocale_Opt_OffX and string.find(msg, TinyTipChatLocale_Opt_OffX, 1, true) then
		ChatSetNum("TinyTipChatLocale_Opt_OffX", msg )
		if TinyTipUtil_GetDB("OffX") then
			TinyTipAnchor_Hook()
		end
	elseif TinyTipChatLocale_Opt_OffY and string.find(msg, TinyTipChatLocale_Opt_OffY, 1, true) then
		ChatSetNum("TinyTipChatLocale_Opt_OffY", msg )
		if TinyTipUtil_GetDB("OffY") then
			TinyTipAnchor_Hook()
		end
	elseif TinyTipChatLocale_Opt_Scale and string.find(msg, TinyTipChatLocale_Opt_Scale, 1, true) then
		ChatSetNum("TinyTipChatLocale_Opt_Scale", msg, true)
		GameTooltip:SetScale( TinyTipUtil_GetDB("Scale") or 1.0) 
		TinyTipUtil_Tooltip:SetScale( TinyTipUtil_GetDB("Scale") or 1.0 )
	elseif string.find(msg, TinyTipChatLocale_Opt_PvPRank, 1, true) then
		ChatSetNum("TinyTipChatLocale_Opt_PvPRank", msg)
	elseif TinyTipChatLocale_Opt_ToP and string.find(msg, TinyTipChatLocale_Opt_ToP, 1, true) then
		ChatSetNum("TinyTipChatLocale_Opt_ToP", msg)
	elseif TinyTipChatLocale_Opt_ToR and string.find(msg, TinyTipChatLocale_Opt_ToR, 1, true) then
		ChatSetNum("TinyTipChatLocale_Opt_ToR", msg)

	-- non saved variable options
	elseif msg == TinyTipChatLocale_Opt_Default then
		TinyTipUtil_Msg( string.format("%s %s %s.", TinyTipChatLocale_DefaultWarning, 
							    SLASH_TINYTIP2, 
							    TinyTipChatLocale_Confirm ) )
	elseif msg == TinyTipChatLocale_Confirm then -- "hidden option"
		TinyTipUtil_DefaultDB()
	elseif msg == TinyTipChatLocale_Opt_Report then
		local k,v
		for k,v in TinyTipChatLocale_ChatMap do
			TinyTipUtil_Msg( string.format("%s %s [|cFFFFCC00%s|r]", v,
						 TinyTipChatLocale_IsSetTo,
						 ( TinyTipUtil_GetDB(k) and type(TinyTipUtil_GetDB(k)) == "boolean" 
						    and TinyTipChatLocale_On ) or 
						 (TinyTipUtil_GetDB(k) and type(TinyTipUtil_GetDB(k)) == "string" and 
						 string.format('"%s"', TinyTipUtil_GetDB(k))) or
						 (type(TinyTipUtil_GetDB(k)) == "number" and TinyTipUtil_GetDB(k)) or 
						 TinyTipChatLocale_Off ) )
		end
	elseif msg and string.len(msg) > 0 then
		TinyTipUtil_Msg( string.format('"|cFFFF2222%s|r" %s', msg, TinyTipChatLocale_NotValidCommand) )
	else
		local k
		TinyTipUtil_Msg( string.format("%s: %s, %s", 
						 TinyTipChatLocale_Options,
						 SLASH_TINYTIP1,
						 SLASH_TINYTIP2 ) )

		-- system options
		TinyTipUtil_Msg( string.format("%s - %s", 
						TinyTipChatLocale_Opt_Default,
						TinyTipChatLocale_Desc_Default) )
		TinyTipUtil_Msg( string.format("%s - %s", 
						TinyTipChatLocale_Opt_Report,
						TinyTipChatLocale_Desc_Report ) )
						
		-- saved variable options
		for k,_ in TinyTipChatLocale_ChatMap do
			if getglobal("TinyTipChatLocale_Opt_" .. k) then
				TinyTipUtil_Msg( string.format("%s - %s",
						getglobal("TinyTipChatLocale_Opt_" .. k),
						getglobal("TinyTipChatLocale_Desc_" .. k) ) )
			end
		end
	end
end

